package javax.swing.plaf.synth;

import java.beans.PropertyChangeEvent;
import java.awt.Font;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import sun.swing.SwingUtilities2;
import java.awt.Shape;
import java.awt.Insets;
import java.awt.Graphics;
import javax.swing.SwingUtilities;
import java.awt.Rectangle;
import java.awt.FontMetrics;
import javax.swing.JProgressBar;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Dimension;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicProgressBarUI;

public class SynthProgressBarUI extends BasicProgressBarUI implements SynthUI, PropertyChangeListener
{
    private SynthStyle style;
    private int progressPadding;
    private boolean rotateText;
    private boolean paintOutsideClip;
    private boolean tileWhenIndeterminate;
    private int tileWidth;
    private Dimension minBarSize;
    private int glowWidth;
    
    public SynthProgressBarUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthProgressBarUI();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.progressBar.addPropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.progressBar.removePropertyChangeListener(this);
    }
    
    @Override
    protected void installDefaults() {
        this.updateStyle(this.progressBar);
    }
    
    private void updateStyle(JProgressBar progressBar) {
        SynthContext context = this.getContext(progressBar, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        this.setCellLength(this.style.getInt(context, "ProgressBar.cellLength", 1));
        this.setCellSpacing(this.style.getInt(context, "ProgressBar.cellSpacing", 0));
        this.progressPadding = this.style.getInt(context, "ProgressBar.progressPadding", 0);
        this.paintOutsideClip = this.style.getBoolean(context, "ProgressBar.paintOutsideClip", false);
        this.rotateText = this.style.getBoolean(context, "ProgressBar.rotateText", false);
        this.tileWhenIndeterminate = this.style.getBoolean(context, "ProgressBar.tileWhenIndeterminate", false);
        this.tileWidth = this.style.getInt(context, "ProgressBar.tileWidth", 15);
        String anObject = (String)this.progressBar.getClientProperty("JComponent.sizeVariant");
        if (anObject != null) {
            if ("large".equals(anObject)) {
                this.tileWidth = (int)((double)this.tileWidth * 1.15);
            }
            else if ("small".equals(anObject)) {
                this.tileWidth = (int)((double)this.tileWidth * 0.857);
            }
            else if ("mini".equals(anObject)) {
                this.tileWidth = (int)((double)this.tileWidth * 0.784);
            }
        }
        this.minBarSize = (Dimension)this.style.get(context, "ProgressBar.minBarSize");
        this.glowWidth = this.style.getInt(context, "ProgressBar.glowWidth", 0);
        context.dispose();
    }
    
    @Override
    protected void uninstallDefaults() {
        SynthContext context = this.getContext(this.progressBar, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        return SynthLookAndFeel.getComponentState(component);
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        if (this.progressBar.isStringPainted() && this.progressBar.getOrientation() == 0) {
            SynthContext context = this.getContext(c);
            FontMetrics fontMetrics = this.progressBar.getFontMetrics(context.getStyle().getFont(context));
            context.dispose();
            return (height - fontMetrics.getAscent() - fontMetrics.getDescent()) / 2 + fontMetrics.getAscent();
        }
        return -1;
    }
    
    @Override
    protected Rectangle getBox(Rectangle rectangle) {
        if (this.tileWhenIndeterminate) {
            return SwingUtilities.calculateInnerArea(this.progressBar, rectangle);
        }
        return super.getBox(rectangle);
    }
    
    @Override
    protected void setAnimationIndex(int n) {
        if (this.paintOutsideClip) {
            if (this.getAnimationIndex() == n) {
                return;
            }
            super.setAnimationIndex(n);
            this.progressBar.repaint();
        }
        else {
            super.setAnimationIndex(n);
        }
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintProgressBarBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight(), this.progressBar.getOrientation());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        JProgressBar progressBar = (JProgressBar)synthContext.getComponent();
        int n = 0;
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        if (!progressBar.isIndeterminate()) {
            Insets insets = progressBar.getInsets();
            double percentComplete = progressBar.getPercentComplete();
            if (percentComplete != 0.0) {
                if (progressBar.getOrientation() == 0) {
                    n = insets.left + this.progressPadding;
                    n2 = insets.top + this.progressPadding;
                    n3 = (int)(percentComplete * (double)(progressBar.getWidth() - (insets.left + this.progressPadding + insets.right + this.progressPadding)));
                    n4 = progressBar.getHeight() - (insets.top + this.progressPadding + insets.bottom + this.progressPadding);
                    if (!SynthLookAndFeel.isLeftToRight(progressBar)) {
                        n = progressBar.getWidth() - insets.right - n3 - this.progressPadding - this.glowWidth;
                    }
                }
                else {
                    n = insets.left + this.progressPadding;
                    n3 = progressBar.getWidth() - (insets.left + this.progressPadding + insets.right + this.progressPadding);
                    n4 = (int)(percentComplete * (double)(progressBar.getHeight() - (insets.top + this.progressPadding + insets.bottom + this.progressPadding)));
                    n2 = progressBar.getHeight() - insets.bottom - n4 - this.progressPadding;
                    if (SynthLookAndFeel.isLeftToRight(progressBar)) {
                        n2 -= this.glowWidth;
                    }
                }
            }
        }
        else {
            this.boxRect = this.getBox(this.boxRect);
            n = this.boxRect.x + this.progressPadding;
            n2 = this.boxRect.y + this.progressPadding;
            n3 = this.boxRect.width - this.progressPadding - this.progressPadding;
            n4 = this.boxRect.height - this.progressPadding - this.progressPadding;
        }
        if (this.tileWhenIndeterminate && progressBar.isIndeterminate()) {
            int n5 = (int)((double)this.getAnimationIndex() / (double)this.getFrameCount() * (double)this.tileWidth);
            Shape clip = graphics.getClip();
            graphics.clipRect(n, n2, n3, n4);
            if (progressBar.getOrientation() == 0) {
                for (int j = n - this.tileWidth + n5; j <= n3; j += this.tileWidth) {
                    synthContext.getPainter().paintProgressBarForeground(synthContext, graphics, j, n2, this.tileWidth, n4, progressBar.getOrientation());
                }
            }
            else {
                for (int i = n2 - n5; i < n4 + this.tileWidth; i += this.tileWidth) {
                    synthContext.getPainter().paintProgressBarForeground(synthContext, graphics, n, i, n3, this.tileWidth, progressBar.getOrientation());
                }
            }
            graphics.setClip(clip);
        }
        else if (this.minBarSize == null || (n3 >= this.minBarSize.width && n4 >= this.minBarSize.height)) {
            synthContext.getPainter().paintProgressBarForeground(synthContext, graphics, n, n2, n3, n4, progressBar.getOrientation());
        }
        if (progressBar.isStringPainted()) {
            this.paintText(synthContext, graphics, progressBar.getString());
        }
    }
    
    protected void paintText(SynthContext synthContext, Graphics g, String s) {
        if (this.progressBar.isStringPainted()) {
            SynthStyle style = synthContext.getStyle();
            Font font = style.getFont(synthContext);
            FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(this.progressBar, g, font);
            int computeStringWidth = style.getGraphicsUtils(synthContext).computeStringWidth(synthContext, font, fontMetrics, s);
            Rectangle bounds = this.progressBar.getBounds();
            if (this.rotateText && this.progressBar.getOrientation() == 1) {
                Graphics2D graphics2D = (Graphics2D)g;
                AffineTransform trans;
                Point point;
                if (this.progressBar.getComponentOrientation().isLeftToRight()) {
                    trans = AffineTransform.getRotateInstance(-1.5707963267948966);
                    point = new Point((bounds.width + fontMetrics.getAscent() - fontMetrics.getDescent()) / 2, (bounds.height + computeStringWidth) / 2);
                }
                else {
                    trans = AffineTransform.getRotateInstance(1.5707963267948966);
                    point = new Point((bounds.width - fontMetrics.getAscent() + fontMetrics.getDescent()) / 2, (bounds.height - computeStringWidth) / 2);
                }
                if (point.x < 0) {
                    return;
                }
                graphics2D.setFont(font.deriveFont(trans));
                graphics2D.setColor(style.getColor(synthContext, ColorType.TEXT_FOREGROUND));
                style.getGraphicsUtils(synthContext).paintText(synthContext, g, s, point.x, point.y, -1);
            }
            else {
                Rectangle rectangle = new Rectangle(bounds.width / 2 - computeStringWidth / 2, (bounds.height - (fontMetrics.getAscent() + fontMetrics.getDescent())) / 2, 0, 0);
                if (rectangle.y < 0) {
                    return;
                }
                g.setColor(style.getColor(synthContext, ColorType.TEXT_FOREGROUND));
                g.setFont(font);
                style.getGraphicsUtils(synthContext).paintText(synthContext, g, s, rectangle.x, rectangle.y, -1);
            }
        }
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintProgressBarBorder(synthContext, graphics, n, n2, n3, n4, this.progressBar.getOrientation());
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent) || "indeterminate".equals(propertyChangeEvent.getPropertyName())) {
            this.updateStyle((JProgressBar)propertyChangeEvent.getSource());
        }
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        Insets insets = this.progressBar.getInsets();
        FontMetrics fontMetrics = this.progressBar.getFontMetrics(this.progressBar.getFont());
        String string = this.progressBar.getString();
        int n = fontMetrics.getHeight() + fontMetrics.getDescent();
        Dimension dimension;
        if (this.progressBar.getOrientation() == 0) {
            dimension = new Dimension(this.getPreferredInnerHorizontal());
            if (this.progressBar.isStringPainted()) {
                if (n > dimension.height) {
                    dimension.height = n;
                }
                int stringWidth2 = SwingUtilities2.stringWidth(this.progressBar, fontMetrics, string);
                if (stringWidth2 > dimension.width) {
                    dimension.width = stringWidth2;
                }
            }
        }
        else {
            dimension = new Dimension(this.getPreferredInnerVertical());
            if (this.progressBar.isStringPainted()) {
                if (n > dimension.width) {
                    dimension.width = n;
                }
                int stringWidth = SwingUtilities2.stringWidth(this.progressBar, fontMetrics, string);
                if (stringWidth > dimension.height) {
                    dimension.height = stringWidth;
                }
            }
        }
        String anObject = (String)this.progressBar.getClientProperty("JComponent.sizeVariant");
        if (anObject != null) {
            if ("large".equals(anObject)) {
                dimension.width = (int)((float)dimension.width * 1.15f);
                dimension.height = (int)((float)dimension.height * 1.15f);
            }
            else if ("small".equals(anObject)) {
                dimension.width = (int)((float)dimension.width * 0.9f);
                dimension.height = (int)((float)dimension.height * 0.9f);
            }
            else if ("mini".equals(anObject)) {
                dimension.width = (int)((float)dimension.width * 0.784f);
                dimension.height = (int)((float)dimension.height * 0.784f);
            }
        }
        dimension.width += insets.left + insets.right;
        dimension.height += insets.top + insets.bottom;
        return dimension;
    }
}
